@@ -1,7 +1,8 @@
 from fontTools.feaLib.error import FeatureLibError
 from fontTools.feaLib.lexer import Lexer, IncludingLexer, NonIncludingLexer
+from fontTools.feaLib.variableScalar import VariableScalar
 from fontTools.misc.encodingTools import getEncoding
-from fontTools.misc.py23 import *
+from fontTools.misc.textTools import bytechr, tobytes, tostr
 import fontTools.feaLib.ast as ast
 import logging
 import os
@@ -44,7 +45,6 @@ class Parser(object):
     def __init__(
         self, featurefile, glyphNames=(), followIncludes=True, includeDir=None, **kwargs
     ):
-
         if "glyphMap" in kwargs:
             from fontTools.misc.loggingTools import deprecateArgument
 
@@ -72,6 +72,7 @@ class Parser(object):
         self.next_token_location_ = None
         lexerClass = IncludingLexer if followIncludes else NonIncludingLexer
         self.lexer_ = lexerClass(featurefile, includeDir=includeDir)
+        self.missing = {}
         self.advance_lexer_(comments=True)
 
     def parse(self):
@@ -101,6 +102,10 @@ class Parser(object):
                 statements.append(self.parse_markClass_())
             elif self.is_cur_keyword_("feature"):
                 statements.append(self.parse_feature_block_())
+            elif self.is_cur_keyword_("conditionset"):
+                statements.append(self.parse_conditionset_())
+            elif self.is_cur_keyword_("variation"):
+                statements.append(self.parse_feature_block_(variation=True))
             elif self.is_cur_keyword_("table"):
                 statements.append(self.parse_table_())
             elif self.is_cur_keyword_("valueRecordDef"):
@@ -120,6 +125,17 @@ class Parser(object):
                     ),
                     self.cur_token_location_,
                 )
+        # Report any missing glyphs at the end of parsing
+        if self.missing:
+            error = [
+                " %s (first found at %s)" % (name, loc)
+                for name, loc in self.missing.items()
+            ]
+            raise FeatureLibError(
+                "The following glyph names are referenced but are missing from the "
+                "glyph set:\n" + ("\n".join(error)),
+                None,
+            )
         return self.doc_
 
     def parse_anchor_(self):
@@ -152,7 +168,7 @@ class Parser(object):
                 location=location,
             )
 
-        x, y = self.expect_number_(), self.expect_number_()
+        x, y = self.expect_number_(variable=True), self.expect_number_(variable=True)
 
         contourpoint = None
         if self.next_token_ == "contourpoint":  # Format B
@@ -314,10 +330,15 @@ class Parser(object):
                 location,
             )
 
-    def parse_glyphclass_(self, accept_glyphname):
+    def parse_glyphclass_(self, accept_glyphname, accept_null=False):
         # Parses a glyph class, either named or anonymous, or (if
-        # ``bool(accept_glyphname)``) a glyph name.
+        # ``bool(accept_glyphname)``) a glyph name. If ``bool(accept_null)`` then
+        # also accept the special NULL glyph.
         if accept_glyphname and self.next_token_type_ in (Lexer.NAME, Lexer.CID):
+            if accept_null and self.next_token_ == "NULL":
+                # If you want a glyph called NULL, you should escape it.
+                self.advance_lexer_()
+                return self.ast.NullGlyph(location=self.cur_token_location_)
             glyph = self.expect_glyph_()
             self.check_glyph_name_in_glyph_set(glyph)
             return self.ast.GlyphName(glyph, location=self.cur_token_location_)
@@ -375,7 +396,8 @@ class Parser(object):
                     self.expect_symbol_("-")
                     range_end = self.expect_cid_()
                     self.check_glyph_name_in_glyph_set(
-                        f"cid{range_start:05d}", f"cid{range_end:05d}",
+                        f"cid{range_start:05d}",
+                        f"cid{range_end:05d}",
                     )
                     glyphs.add_cid_range(
                         range_start,
@@ -467,37 +489,67 @@ class Parser(object):
             assert lookups == []
             return ([], prefix, [None] * len(prefix), values, [], hasMarks)
         else:
-            assert not any(values[: len(prefix)]), values
-            format1 = values[len(prefix) :][: len(glyphs)]
-            format2 = values[(len(prefix) + len(glyphs)) :][: len(suffix)]
-            values = (
-                format2
-                if format2 and isinstance(format2[0], self.ast.ValueRecord)
-                else format1
-            )
+            if any(values[: len(prefix)]):
+                raise FeatureLibError(
+                    "Positioning cannot be applied in the bactrack glyph sequence, "
+                    "before the marked glyph sequence.",
+                    self.cur_token_location_,
+                )
+            marked_values = values[len(prefix) : len(prefix) + len(glyphs)]
+            if any(marked_values):
+                if any(values[len(prefix) + len(glyphs) :]):
+                    raise FeatureLibError(
+                        "Positioning values are allowed only in the marked glyph "
+                        "sequence, or after the final glyph node when only one glyph "
+                        "node is marked.",
+                        self.cur_token_location_,
+                    )
+                values = marked_values
+            elif values and values[-1]:
+                if len(glyphs) > 1 or any(values[:-1]):
+                    raise FeatureLibError(
+                        "Positioning values are allowed only in the marked glyph "
+                        "sequence, or after the final glyph node when only one glyph "
+                        "node is marked.",
+                        self.cur_token_location_,
+                    )
+                values = values[-1:]
+            elif any(values):
+                raise FeatureLibError(
+                    "Positioning values are allowed only in the marked glyph "
+                    "sequence, or after the final glyph node when only one glyph "
+                    "node is marked.",
+                    self.cur_token_location_,
+                )
             return (prefix, glyphs, lookups, values, suffix, hasMarks)
 
-    def parse_chain_context_(self):
+    def parse_ignore_glyph_pattern_(self, sub):
         location = self.cur_token_location_
         prefix, glyphs, lookups, values, suffix, hasMarks = self.parse_glyph_pattern_(
             vertical=False
         )
-        chainContext = [(prefix, glyphs, suffix)]
-        hasLookups = any(lookups)
+        if any(lookups):
+            raise FeatureLibError(
+                f'No lookups can be specified for "ignore {sub}"', location
+            )
+        if not hasMarks:
+            error = FeatureLibError(
+                f'Ambiguous "ignore {sub}", there should be least one marked glyph',
+                location,
+            )
+            log.warning(str(error))
+            suffix, glyphs = glyphs[1:], glyphs[0:1]
+        chainContext = (prefix, glyphs, suffix)
+        return chainContext
+
+    def parse_ignore_context_(self, sub):
+        location = self.cur_token_location_
+        chainContext = [self.parse_ignore_glyph_pattern_(sub)]
         while self.next_token_ == ",":
             self.expect_symbol_(",")
-            (
-                prefix,
-                glyphs,
-                lookups,
-                values,
-                suffix,
-                hasMarks,
-            ) = self.parse_glyph_pattern_(vertical=False)
-            chainContext.append((prefix, glyphs, suffix))
-            hasLookups = hasLookups or any(lookups)
+            chainContext.append(self.parse_ignore_glyph_pattern_(sub))
         self.expect_symbol_(";")
-        return chainContext, hasLookups
+        return chainContext
 
     def parse_ignore_(self):
         # Parses an ignore sub/pos rule.
@@ -505,18 +557,10 @@ class Parser(object):
         location = self.cur_token_location_
         self.advance_lexer_()
         if self.cur_token_ in ["substitute", "sub"]:
-            chainContext, hasLookups = self.parse_chain_context_()
-            if hasLookups:
-                raise FeatureLibError(
-                    'No lookups can be specified for "ignore sub"', location
-                )
+            chainContext = self.parse_ignore_context_("sub")
             return self.ast.IgnoreSubstStatement(chainContext, location=location)
         if self.cur_token_ in ["position", "pos"]:
-            chainContext, hasLookups = self.parse_chain_context_()
-            if hasLookups:
-                raise FeatureLibError(
-                    'No lookups can be specified for "ignore pos"', location
-                )
+            chainContext = self.parse_ignore_context_("pos")
             return self.ast.IgnorePosStatement(chainContext, location=location)
         raise FeatureLibError(
             'Expected "substitute" or "position"', self.cur_token_location_
@@ -558,9 +602,9 @@ class Parser(object):
         assert self.is_cur_keyword_("LigatureCaretByPos")
         location = self.cur_token_location_
         glyphs = self.parse_glyphclass_(accept_glyphname=True)
-        carets = [self.expect_number_()]
+        carets = [self.expect_number_(variable=True)]
         while self.next_token_ != ";":
-            carets.append(self.expect_number_())
+            carets.append(self.expect_number_(variable=True))
         self.expect_symbol_(";")
         return self.ast.LigatureCaretByPosStatement(glyphs, carets, location=location)
 
@@ -650,6 +694,10 @@ class Parser(object):
         assert self.is_cur_keyword_("markClass")
         location = self.cur_token_location_
         glyphs = self.parse_glyphclass_(accept_glyphname=True)
+        if not glyphs.glyphSet():
+            raise FeatureLibError(
+                "Empty glyph class in mark class definition", location
+            )
         anchor = self.parse_anchor_()
         name = self.expect_class_name_()
         self.expect_symbol_(";")
@@ -804,7 +852,7 @@ class Parser(object):
         if self.next_token_ == "by":
             keyword = self.expect_keyword_("by")
             while self.next_token_ != ";":
-                gc = self.parse_glyphclass_(accept_glyphname=True)
+                gc = self.parse_glyphclass_(accept_glyphname=True, accept_null=True)
                 new.append(gc)
         elif self.next_token_ == "from":
             keyword = self.expect_keyword_("from")
@@ -837,6 +885,11 @@ class Parser(object):
 
         num_lookups = len([l for l in lookups if l is not None])
 
+        is_deletion = False
+        if len(new) == 1 and isinstance(new[0], ast.NullGlyph):
+            new = []  # Deletion
+            is_deletion = True
+
         # GSUB lookup type 1: Single substitution.
         # Format A: "substitute a by a.sc;"
         # Format B: "substitute [one.fitted one.oldstyle] by one;"
@@ -857,21 +910,41 @@ class Parser(object):
                 old, new, old_prefix, old_suffix, forceChain=hasMarks, location=location
             )
 
-        # GSUB lookup type 2: Multiple substitution.
-        # Format: "substitute f_f_i by f f i;"
-        if (
-            not reverse
-            and len(old) == 1
-            and len(old[0].glyphSet()) == 1
-            and len(new) > 1
-            and max([len(n.glyphSet()) for n in new]) == 1
-            and num_lookups == 0
-        ):
+        # Glyph deletion, built as GSUB lookup type 2: Multiple substitution
+        # with empty replacement.
+        if is_deletion and len(old) == 1 and num_lookups == 0:
             return self.ast.MultipleSubstStatement(
                 old_prefix,
-                tuple(old[0].glyphSet())[0],
+                old[0],
                 old_suffix,
-                tuple([list(n.glyphSet())[0] for n in new]),
+                (),
+                forceChain=hasMarks,
+                location=location,
+            )
+
+        # GSUB lookup type 2: Multiple substitution.
+        # Format: "substitute f_f_i by f f i;"
+        #
+        # GlyphsApp introduces two additional formats:
+        # Format 1: "substitute [f_i f_l] by [f f] [i l];"
+        # Format 2: "substitute [f_i f_l] by f [i l];"
+        # http://handbook.glyphsapp.com/en/layout/multiple-substitution-with-classes/
+        if not reverse and len(old) == 1 and len(new) > 1 and num_lookups == 0:
+            count = len(old[0].glyphSet())
+            for n in new:
+                if not list(n.glyphSet()):
+                    raise FeatureLibError("Empty class in replacement", location)
+                if len(n.glyphSet()) != 1 and len(n.glyphSet()) != count:
+                    raise FeatureLibError(
+                        f'Expected a glyph class with 1 or {count} elements after "by", '
+                        f"but found a glyph class with {len(n.glyphSet())} elements",
+                        location,
+                    )
+            return self.ast.MultipleSubstStatement(
+                old_prefix,
+                old[0],
+                old_suffix,
+                new,
                 forceChain=hasMarks,
                 location=location,
             )
@@ -936,7 +1009,7 @@ class Parser(object):
             )
 
         # If there are remaining glyphs to parse, this is an invalid GSUB statement
-        if len(new) != 0:
+        if len(new) != 0 or is_deletion:
             raise FeatureLibError("Invalid substitution statement", location)
 
         # GSUB lookup type 6: Chaining contextual substitution.
@@ -958,8 +1031,8 @@ class Parser(object):
         location = self.cur_token_location_
         DesignSize = self.expect_decipoint_()
         SubfamilyID = self.expect_number_()
-        RangeStart = 0
-        RangeEnd = 0
+        RangeStart = 0.0
+        RangeEnd = 0.0
         if self.next_token_type_ in (Lexer.NUMBER, Lexer.FLOAT) or SubfamilyID != 0:
             RangeStart = self.expect_decipoint_()
             RangeEnd = self.expect_decipoint_()
@@ -990,6 +1063,7 @@ class Parser(object):
             "name": self.parse_table_name_,
             "BASE": self.parse_table_BASE_,
             "OS/2": self.parse_table_OS_2_,
+            "STAT": self.parse_table_STAT_,
         }.get(name)
         if handler:
             handler(table)
@@ -1149,6 +1223,35 @@ class Parser(object):
         unescaped = self.unescape_string_(string, encoding)
         return platformID, platEncID, langID, unescaped
 
+    def parse_stat_name_(self):
+        platEncID = None
+        langID = None
+        if self.next_token_type_ in Lexer.NUMBERS:
+            platformID = self.expect_any_number_()
+            location = self.cur_token_location_
+            if platformID not in (1, 3):
+                raise FeatureLibError("Expected platform id 1 or 3", location)
+            if self.next_token_type_ in Lexer.NUMBERS:
+                platEncID = self.expect_any_number_()
+                langID = self.expect_any_number_()
+        else:
+            platformID = 3
+            location = self.cur_token_location_
+
+        if platformID == 1:  # Macintosh
+            platEncID = platEncID or 0  # Roman
+            langID = langID or 0  # English
+        else:  # 3, Windows
+            platEncID = platEncID or 1  # Unicode
+            langID = langID or 0x0409  # English
+
+        string = self.expect_string_()
+        encoding = getEncoding(platformID, platEncID, langID)
+        if encoding is None:
+            raise FeatureLibError("Unsupported encoding", location)
+        unescaped = self.unescape_string_(string, encoding)
+        return platformID, platEncID, langID, unescaped
+
     def parse_nameid_(self):
         assert self.cur_token_ == "nameid", self.cur_token_
         location, nameID = self.cur_token_location_, self.expect_any_number_()
@@ -1156,14 +1259,6 @@ class Parser(object):
             raise FeatureLibError(
                 "Name id value cannot be greater than 32767", self.cur_token_location_
             )
-        if 1 <= nameID <= 6:
-            log.warning(
-                "Name id %d cannot be set from the feature file. "
-                "Ignoring record" % nameID
-            )
-            self.parse_name_()  # skip to the next record
-            return None
-
         platformID, platEncID, langID, string = self.parse_name_()
         return self.ast.NameRecord(
             nameID, platformID, platEncID, langID, string, location=location
@@ -1179,12 +1274,12 @@ class Parser(object):
         # We convert surrogates to actual Unicode by round-tripping through
         # Python's UTF-16 codec in a special mode.
         utf16 = tobytes(s, "utf_16_be", "surrogatepass")
-        return tounicode(utf16, "utf_16_be")
+        return tostr(utf16, "utf_16_be")
 
     @staticmethod
     def unescape_unichr_(match):
         n = match.group(0)[1:]
-        return unichr(int(n, 16))
+        return chr(int(n, 16))
 
     @staticmethod
     def unescape_byte_(match, encoding):
@@ -1270,6 +1365,198 @@ class Parser(object):
             elif self.cur_token_ == ";":
                 continue
 
+    def parse_STAT_ElidedFallbackName(self):
+        assert self.is_cur_keyword_("ElidedFallbackName")
+        self.expect_symbol_("{")
+        names = []
+        while self.next_token_ != "}" or self.cur_comments_:
+            self.advance_lexer_()
+            if self.is_cur_keyword_("name"):
+                platformID, platEncID, langID, string = self.parse_stat_name_()
+                nameRecord = self.ast.STATNameStatement(
+                    "stat",
+                    platformID,
+                    platEncID,
+                    langID,
+                    string,
+                    location=self.cur_token_location_,
+                )
+                names.append(nameRecord)
+            else:
+                if self.cur_token_ != ";":
+                    raise FeatureLibError(
+                        f"Unexpected token {self.cur_token_} " f"in ElidedFallbackName",
+                        self.cur_token_location_,
+                    )
+        self.expect_symbol_("}")
+        if not names:
+            raise FeatureLibError('Expected "name"', self.cur_token_location_)
+        return names
+
+    def parse_STAT_design_axis(self):
+        assert self.is_cur_keyword_("DesignAxis")
+        names = []
+        axisTag = self.expect_tag_()
+        if (
+            axisTag not in ("ital", "opsz", "slnt", "wdth", "wght")
+            and not axisTag.isupper()
+        ):
+            log.warning(f"Unregistered axis tag {axisTag} should be uppercase.")
+        axisOrder = self.expect_number_()
+        self.expect_symbol_("{")
+        while self.next_token_ != "}" or self.cur_comments_:
+            self.advance_lexer_()
+            if self.cur_token_type_ is Lexer.COMMENT:
+                continue
+            elif self.is_cur_keyword_("name"):
+                location = self.cur_token_location_
+                platformID, platEncID, langID, string = self.parse_stat_name_()
+                name = self.ast.STATNameStatement(
+                    "stat", platformID, platEncID, langID, string, location=location
+                )
+                names.append(name)
+            elif self.cur_token_ == ";":
+                continue
+            else:
+                raise FeatureLibError(
+                    f'Expected "name", got {self.cur_token_}', self.cur_token_location_
+                )
+
+        self.expect_symbol_("}")
+        return self.ast.STATDesignAxisStatement(
+            axisTag, axisOrder, names, self.cur_token_location_
+        )
+
+    def parse_STAT_axis_value_(self):
+        assert self.is_cur_keyword_("AxisValue")
+        self.expect_symbol_("{")
+        locations = []
+        names = []
+        flags = 0
+        while self.next_token_ != "}" or self.cur_comments_:
+            self.advance_lexer_(comments=True)
+            if self.cur_token_type_ is Lexer.COMMENT:
+                continue
+            elif self.is_cur_keyword_("name"):
+                location = self.cur_token_location_
+                platformID, platEncID, langID, string = self.parse_stat_name_()
+                name = self.ast.STATNameStatement(
+                    "stat", platformID, platEncID, langID, string, location=location
+                )
+                names.append(name)
+            elif self.is_cur_keyword_("location"):
+                location = self.parse_STAT_location()
+                locations.append(location)
+            elif self.is_cur_keyword_("flag"):
+                flags = self.expect_stat_flags()
+            elif self.cur_token_ == ";":
+                continue
+            else:
+                raise FeatureLibError(
+                    f"Unexpected token {self.cur_token_} " f"in AxisValue",
+                    self.cur_token_location_,
+                )
+        self.expect_symbol_("}")
+        if not names:
+            raise FeatureLibError('Expected "Axis Name"', self.cur_token_location_)
+        if not locations:
+            raise FeatureLibError('Expected "Axis location"', self.cur_token_location_)
+        if len(locations) > 1:
+            for location in locations:
+                if len(location.values) > 1:
+                    raise FeatureLibError(
+                        "Only one value is allowed in a "
+                        "Format 4 Axis Value Record, but "
+                        f"{len(location.values)} were found.",
+                        self.cur_token_location_,
+                    )
+            format4_tags = []
+            for location in locations:
+                tag = location.tag
+                if tag in format4_tags:
+                    raise FeatureLibError(
+                        f"Axis tag {tag} already " "defined.", self.cur_token_location_
+                    )
+                format4_tags.append(tag)
+
+        return self.ast.STATAxisValueStatement(
+            names, locations, flags, self.cur_token_location_
+        )
+
+    def parse_STAT_location(self):
+        values = []
+        tag = self.expect_tag_()
+        if len(tag.strip()) != 4:
+            raise FeatureLibError(
+                f"Axis tag {self.cur_token_} must be 4 " "characters",
+                self.cur_token_location_,
+            )
+
+        while self.next_token_ != ";":
+            if self.next_token_type_ is Lexer.FLOAT:
+                value = self.expect_float_()
+                values.append(value)
+            elif self.next_token_type_ is Lexer.NUMBER:
+                value = self.expect_number_()
+                values.append(value)
+            else:
+                raise FeatureLibError(
+                    f'Unexpected value "{self.next_token_}". '
+                    "Expected integer or float.",
+                    self.next_token_location_,
+                )
+        if len(values) == 3:
+            nominal, min_val, max_val = values
+            if nominal < min_val or nominal > max_val:
+                raise FeatureLibError(
+                    f"Default value {nominal} is outside "
+                    f"of specified range "
+                    f"{min_val}-{max_val}.",
+                    self.next_token_location_,
+                )
+        return self.ast.AxisValueLocationStatement(tag, values)
+
+    def parse_table_STAT_(self, table):
+        statements = table.statements
+        design_axes = []
+        while self.next_token_ != "}" or self.cur_comments_:
+            self.advance_lexer_(comments=True)
+            if self.cur_token_type_ is Lexer.COMMENT:
+                statements.append(
+                    self.ast.Comment(self.cur_token_, location=self.cur_token_location_)
+                )
+            elif self.cur_token_type_ is Lexer.NAME:
+                if self.is_cur_keyword_("ElidedFallbackName"):
+                    names = self.parse_STAT_ElidedFallbackName()
+                    statements.append(self.ast.ElidedFallbackName(names))
+                elif self.is_cur_keyword_("ElidedFallbackNameID"):
+                    value = self.expect_number_()
+                    statements.append(self.ast.ElidedFallbackNameID(value))
+                    self.expect_symbol_(";")
+                elif self.is_cur_keyword_("DesignAxis"):
+                    designAxis = self.parse_STAT_design_axis()
+                    design_axes.append(designAxis.tag)
+                    statements.append(designAxis)
+                    self.expect_symbol_(";")
+                elif self.is_cur_keyword_("AxisValue"):
+                    axisValueRecord = self.parse_STAT_axis_value_()
+                    for location in axisValueRecord.locations:
+                        if location.tag not in design_axes:
+                            # Tag must be defined in a DesignAxis before it
+                            # can be referenced
+                            raise FeatureLibError(
+                                "DesignAxis not defined for " f"{location.tag}.",
+                                self.cur_token_location_,
+                            )
+                    statements.append(axisValueRecord)
+                    self.expect_symbol_(";")
+                else:
+                    raise FeatureLibError(
+                        f"Unexpected token {self.cur_token_}", self.cur_token_location_
+                    )
+            elif self.cur_token_ == ";":
+                continue
+
     def parse_base_tag_list_(self):
         # Parses BASE table entries. (See `section 9.a <https://adobe-type-tools.github.io/afdko/OpenTypeFeatureFileSpecification.html#9.a>`_)
         assert self.cur_token_ in (
@@ -1316,11 +1603,20 @@ class Parser(object):
         return result
 
     def is_next_value_(self):
-        return self.next_token_type_ is Lexer.NUMBER or self.next_token_ == "<"
+        return (
+            self.next_token_type_ is Lexer.NUMBER
+            or self.next_token_ == "<"
+            or self.next_token_ == "("
+        )
 
     def parse_valuerecord_(self, vertical):
-        if self.next_token_type_ is Lexer.NUMBER:
-            number, location = self.expect_number_(), self.cur_token_location_
+        if (
+            self.next_token_type_ is Lexer.SYMBOL and self.next_token_ == "("
+        ) or self.next_token_type_ is Lexer.NUMBER:
+            number, location = (
+                self.expect_number_(variable=True),
+                self.cur_token_location_,
+            )
             if vertical:
                 val = self.ast.ValueRecord(
                     yAdvance=number, vertical=vertical, location=location
@@ -1347,10 +1643,10 @@ class Parser(object):
             xAdvance, yAdvance = (value.xAdvance, value.yAdvance)
         else:
             xPlacement, yPlacement, xAdvance, yAdvance = (
-                self.expect_number_(),
-                self.expect_number_(),
-                self.expect_number_(),
-                self.expect_number_(),
+                self.expect_number_(variable=True),
+                self.expect_number_(variable=True),
+                self.expect_number_(variable=True),
+                self.expect_number_(variable=True),
             )
 
         if self.next_token_ == "<":
@@ -1410,8 +1706,11 @@ class Parser(object):
         self.expect_symbol_(";")
         return self.ast.LanguageSystemStatement(script, language, location=location)
 
-    def parse_feature_block_(self):
-        assert self.cur_token_ == "feature"
+    def parse_feature_block_(self, variation=False):
+        if variation:
+            assert self.cur_token_ == "variation"
+        else:
+            assert self.cur_token_ == "feature"
         location = self.cur_token_location_
         tag = self.expect_tag_()
         vertical = tag in {"vkrn", "vpal", "vhal", "valt"}
@@ -1426,14 +1725,22 @@ class Parser(object):
         elif tag == "size":
             size_feature = True
 
+        if variation:
+            conditionset = self.expect_name_()
+
         use_extension = False
         if self.next_token_ == "useExtension":
             self.expect_keyword_("useExtension")
             use_extension = True
 
-        block = self.ast.FeatureBlock(
-            tag, use_extension=use_extension, location=location
-        )
+        if variation:
+            block = self.ast.VariationBlock(
+                tag, conditionset, use_extension=use_extension, location=location
+            )
+        else:
+            block = self.ast.FeatureBlock(
+                tag, use_extension=use_extension, location=location
+            )
         self.parse_block_(block, vertical, stylisticset, size_feature, cv_feature)
         return block
 
@@ -1446,7 +1753,8 @@ class Parser(object):
 
     def parse_featureNames_(self, tag):
         """Parses a ``featureNames`` statement found in stylistic set features.
-        See section `8.c <https://adobe-type-tools.github.io/afdko/OpenTypeFeatureFileSpecification.html#8.c>`_."""
+        See section `8.c <https://adobe-type-tools.github.io/afdko/OpenTypeFeatureFileSpecification.html#8.c>`_.
+        """
         assert self.cur_token_ == "featureNames", self.cur_token_
         block = self.ast.NestedBlock(
             tag, self.cur_token_, location=self.cur_token_location_
@@ -1581,6 +1889,43 @@ class Parser(object):
             raise FeatureLibError("Font revision numbers must be positive", location)
         return self.ast.FontRevisionStatement(version, location=location)
 
+    def parse_conditionset_(self):
+        name = self.expect_name_()
+
+        conditions = {}
+        self.expect_symbol_("{")
+
+        while self.next_token_ != "}":
+            self.advance_lexer_()
+            if self.cur_token_type_ is not Lexer.NAME:
+                raise FeatureLibError("Expected an axis name", self.cur_token_location_)
+
+            axis = self.cur_token_
+            if axis in conditions:
+                raise FeatureLibError(
+                    f"Repeated condition for axis {axis}", self.cur_token_location_
+                )
+
+            if self.next_token_type_ is Lexer.FLOAT:
+                min_value = self.expect_float_()
+            elif self.next_token_type_ is Lexer.NUMBER:
+                min_value = self.expect_number_(variable=False)
+
+            if self.next_token_type_ is Lexer.FLOAT:
+                max_value = self.expect_float_()
+            elif self.next_token_type_ is Lexer.NUMBER:
+                max_value = self.expect_number_(variable=False)
+            self.expect_symbol_(";")
+
+            conditions[axis] = (min_value, max_value)
+
+        self.expect_symbol_("}")
+
+        finalname = self.expect_name_()
+        if finalname != name:
+            raise FeatureLibError('Expected "%s"' % name, self.cur_token_location_)
+        return self.ast.ConditionsetStatement(name, conditions)
+
     def parse_block_(
         self, block, vertical, stylisticset=None, size_feature=False, cv_feature=None
     ):
@@ -1738,19 +2083,18 @@ class Parser(object):
         raise FeatureLibError("Expected a glyph name or CID", self.cur_token_location_)
 
     def check_glyph_name_in_glyph_set(self, *names):
-        """Raises if glyph name (just `start`) or glyph names of a
-        range (`start` and `end`) are not in the glyph set.
+        """Adds a glyph name (just `start`) or glyph names of a
+        range (`start` and `end`) which are not in the glyph set
+        to the "missing list" for future error reporting.
 
         If no glyph set is present, does nothing.
         """
         if self.glyphNames_:
-            missing = [name for name in names if name not in self.glyphNames_]
-            if missing:
-                raise FeatureLibError(
-                    "The following glyph names are referenced but are missing from the "
-                    f"glyph set: {', '.join(missing)}",
-                    self.cur_token_location_,
-                )
+            for name in names:
+                if name in self.glyphNames_:
+                    continue
+                if name not in self.missing:
+                    self.missing[name] = self.cur_token_location_
 
     def expect_markClass_reference_(self):
         name = self.expect_class_name_()
@@ -1771,7 +2115,7 @@ class Parser(object):
             raise FeatureLibError("Expected a tag", self.cur_token_location_)
         if len(self.cur_token_) > 4:
             raise FeatureLibError(
-                "Tags can not be longer than 4 characters", self.cur_token_location_
+                "Tags cannot be longer than 4 characters", self.cur_token_location_
             )
         return (self.cur_token_ + "    ")[:4]
 
@@ -1811,12 +2155,51 @@ class Parser(object):
             return self.cur_token_
         raise FeatureLibError("Expected a name", self.cur_token_location_)
 
-    def expect_number_(self):
+    def expect_number_(self, variable=False):
         self.advance_lexer_()
         if self.cur_token_type_ is Lexer.NUMBER:
             return self.cur_token_
+        if variable and self.cur_token_type_ is Lexer.SYMBOL and self.cur_token_ == "(":
+            return self.expect_variable_scalar_()
         raise FeatureLibError("Expected a number", self.cur_token_location_)
 
+    def expect_variable_scalar_(self):
+        self.advance_lexer_()  # "("
+        scalar = VariableScalar()
+        while True:
+            if self.cur_token_type_ == Lexer.SYMBOL and self.cur_token_ == ")":
+                break
+            location, value = self.expect_master_()
+            scalar.add_value(location, value)
+        return scalar
+
+    def expect_master_(self):
+        location = {}
+        while True:
+            if self.cur_token_type_ is not Lexer.NAME:
+                raise FeatureLibError("Expected an axis name", self.cur_token_location_)
+            axis = self.cur_token_
+            self.advance_lexer_()
+            if not (self.cur_token_type_ is Lexer.SYMBOL and self.cur_token_ == "="):
+                raise FeatureLibError(
+                    "Expected an equals sign", self.cur_token_location_
+                )
+            value = self.expect_number_()
+            location[axis] = value
+            if self.next_token_type_ is Lexer.NAME and self.next_token_[0] == ":":
+                # Lexer has just read the value as a glyph name. We'll correct it later
+                break
+            self.advance_lexer_()
+            if not (self.cur_token_type_ is Lexer.SYMBOL and self.cur_token_ == ","):
+                raise FeatureLibError(
+                    "Expected an comma or an equals sign", self.cur_token_location_
+                )
+            self.advance_lexer_()
+        self.advance_lexer_()
+        value = int(self.cur_token_[1:])
+        self.advance_lexer_()
+        return location, value
+
     def expect_any_number_(self):
         self.advance_lexer_()
         if self.cur_token_type_ in Lexer.NUMBERS:
@@ -1843,6 +2226,32 @@ class Parser(object):
                 "Expected an integer or floating-point number", self.cur_token_location_
             )
 
+    def expect_stat_flags(self):
+        value = 0
+        flags = {
+            "OlderSiblingFontAttribute": 1,
+            "ElidableAxisValueName": 2,
+        }
+        while self.next_token_ != ";":
+            if self.next_token_ in flags:
+                name = self.expect_name_()
+                value = value | flags[name]
+            else:
+                raise FeatureLibError(
+                    f"Unexpected STAT flag {self.cur_token_}", self.cur_token_location_
+                )
+        return value
+
+    def expect_stat_values_(self):
+        if self.next_token_type_ == Lexer.FLOAT:
+            return self.expect_float_()
+        elif self.next_token_type_ is Lexer.NUMBER:
+            return self.expect_number_()
+        else:
+            raise FeatureLibError(
+                "Expected an integer or floating-point number", self.cur_token_location_
+            )
+
     def expect_string_(self):
         self.advance_lexer_()
         if self.cur_token_type_ is Lexer.STRING:
