@@ -4,16 +4,17 @@ https://docs.microsoft.com/en-us/typography/opentype/spec/chapter2#featurevariat
 NOTE: The API is experimental and subject to change.
 """
 from fontTools.misc.dictTools import hashdict
-from fontTools.misc.intTools import popCount
+from fontTools.misc.intTools import bit_count
 from fontTools.ttLib import newTable
 from fontTools.ttLib.tables import otTables as ot
+from fontTools.ttLib.ttVisitor import TTVisitor
 from fontTools.otlLib.builder import buildLookup, buildSingleSubstSubtable
 from collections import OrderedDict
 
-from .errors import VarLibValidationError
+from .errors import VarLibError, VarLibValidationError
 
 
-def addFeatureVariations(font, conditionalSubstitutions, featureTag='rvrn'):
+def addFeatureVariations(font, conditionalSubstitutions, featureTag="rvrn"):
     """Add conditional substitutions to a Variable Font.
 
     The `conditionalSubstitutions` argument is a list of (Region, Substitutions)
@@ -42,11 +43,80 @@ def addFeatureVariations(font, conditionalSubstitutions, featureTag='rvrn'):
     # ... ]
     # >>> addFeatureVariations(f, condSubst)
     # >>> f.save(dstPath)
+
+    The `featureTag` parameter takes either a str or a iterable of str (the single str
+    is kept for backwards compatibility), and defines which feature(s) will be
+    associated with the feature variations.
+    Note, if this is "rvrn", then the substitution lookup will be inserted at the
+    beginning of the lookup list so that it is processed before others, otherwise
+    for any other feature tags it will be appended last.
     """
 
-    addFeatureVariationsRaw(font,
-                            overlayFeatureVariations(conditionalSubstitutions),
-                            featureTag)
+    # process first when "rvrn" is the only listed tag
+    featureTags = [featureTag] if isinstance(featureTag, str) else sorted(featureTag)
+    processLast = "rvrn" not in featureTags or len(featureTags) > 1
+
+    _checkSubstitutionGlyphsExist(
+        glyphNames=set(font.getGlyphOrder()),
+        substitutions=conditionalSubstitutions,
+    )
+
+    substitutions = overlayFeatureVariations(conditionalSubstitutions)
+
+    # turn substitution dicts into tuples of tuples, so they are hashable
+    conditionalSubstitutions, allSubstitutions = makeSubstitutionsHashable(
+        substitutions
+    )
+    if "GSUB" not in font:
+        font["GSUB"] = buildGSUB()
+    else:
+        existingTags = _existingVariableFeatures(font["GSUB"].table).intersection(
+            featureTags
+        )
+        if existingTags:
+            raise VarLibError(
+                f"FeatureVariations already exist for feature tag(s): {existingTags}"
+            )
+
+    # setup lookups
+    lookupMap = buildSubstitutionLookups(
+        font["GSUB"].table, allSubstitutions, processLast
+    )
+
+    # addFeatureVariationsRaw takes a list of
+    #  ( {condition}, [ lookup indices ] )
+    # so rearrange our lookups to match
+    conditionsAndLookups = []
+    for conditionSet, substitutions in conditionalSubstitutions:
+        conditionsAndLookups.append(
+            (conditionSet, [lookupMap[s] for s in substitutions])
+        )
+
+    addFeatureVariationsRaw(font, font["GSUB"].table, conditionsAndLookups, featureTags)
+
+
+def _existingVariableFeatures(table):
+    existingFeatureVarsTags = set()
+    if hasattr(table, "FeatureVariations") and table.FeatureVariations is not None:
+        features = table.FeatureList.FeatureRecord
+        for fvr in table.FeatureVariations.FeatureVariationRecord:
+            for ftsr in fvr.FeatureTableSubstitution.SubstitutionRecord:
+                existingFeatureVarsTags.add(features[ftsr.FeatureIndex].FeatureTag)
+    return existingFeatureVarsTags
+
+
+def _checkSubstitutionGlyphsExist(glyphNames, substitutions):
+    referencedGlyphNames = set()
+    for _, substitution in substitutions:
+        referencedGlyphNames |= substitution.keys()
+        referencedGlyphNames |= set(substitution.values())
+    missing = referencedGlyphNames - glyphNames
+    if missing:
+        raise VarLibValidationError(
+            "Missing glyphs are referenced in conditional substitution rules:"
+            f" {', '.join(missing)}"
+        )
+
 
 def overlayFeatureVariations(conditionalSubstitutions):
     """Compute overlaps between all conditional substitutions.
@@ -76,30 +146,33 @@ def overlayFeatureVariations(conditionalSubstitutions):
     substitution dictionaries.  These dictionaries are not merged to allow data
     sharing when they are converted into font tables.
 
-    Example:
-    >>> condSubst = [
-    ...     # A list of (Region, Substitution) tuples.
-    ...     ([{"wght": (0.5, 1.0)}], {"dollar": "dollar.rvrn"}),
-    ...     ([{"wght": (0.5, 1.0)}], {"dollar": "dollar.rvrn"}),
-    ...     ([{"wdth": (0.5, 1.0)}], {"cent": "cent.rvrn"}),
-    ... ]
-    >>> from pprint import pprint
-    >>> pprint(overlayFeatureVariations(condSubst))
-    [({'wdth': (0.5, 1.0), 'wght': (0.5, 1.0)},
-      [{'dollar': 'dollar.rvrn'}, {'cent': 'cent.rvrn'}]),
-     ({'wdth': (0.5, 1.0)}, [{'cent': 'cent.rvrn'}]),
-     ({'wght': (0.5, 1.0)}, [{'dollar': 'dollar.rvrn'}])]
+    Example::
+
+        >>> condSubst = [
+        ...     # A list of (Region, Substitution) tuples.
+        ...     ([{"wght": (0.5, 1.0)}], {"dollar": "dollar.rvrn"}),
+        ...     ([{"wght": (0.5, 1.0)}], {"dollar": "dollar.rvrn"}),
+        ...     ([{"wdth": (0.5, 1.0)}], {"cent": "cent.rvrn"}),
+        ...     ([{"wght": (0.5, 1.0), "wdth": (-1, 1.0)}], {"dollar": "dollar.rvrn"}),
+        ... ]
+        >>> from pprint import pprint
+        >>> pprint(overlayFeatureVariations(condSubst))
+        [({'wdth': (0.5, 1.0), 'wght': (0.5, 1.0)},
+          [{'dollar': 'dollar.rvrn'}, {'cent': 'cent.rvrn'}]),
+         ({'wdth': (0.5, 1.0)}, [{'cent': 'cent.rvrn'}]),
+         ({'wght': (0.5, 1.0)}, [{'dollar': 'dollar.rvrn'}])]
+
     """
 
     # Merge same-substitutions rules, as this creates fewer number oflookups.
     merged = OrderedDict()
-    for value,key in conditionalSubstitutions:
+    for value, key in conditionalSubstitutions:
         key = hashdict(key)
         if key in merged:
             merged[key].extend(value)
         else:
             merged[key] = value
-    conditionalSubstitutions = [(v,dict(k)) for k,v in merged.items()]
+    conditionalSubstitutions = [(v, dict(k)) for k, v in merged.items()]
     del merged
 
     # Merge same-region rules, as this is cheaper.
@@ -108,9 +181,13 @@ def overlayFeatureVariations(conditionalSubstitutions):
     # Reversing is such that earlier entries win in case of conflicting substitution
     # rules for the same region.
     merged = OrderedDict()
-    for key,value in reversed(conditionalSubstitutions):
-        key = tuple(sorted((hashdict(cleanupBox(k)) for k in key),
-                           key=lambda d: tuple(sorted(d.items()))))
+    for key, value in reversed(conditionalSubstitutions):
+        key = tuple(
+            sorted(
+                (hashdict(cleanupBox(k)) for k in key),
+                key=lambda d: tuple(sorted(d.items())),
+            )
+        )
         if key in merged:
             merged[key].update(value)
         else:
@@ -121,35 +198,38 @@ def overlayFeatureVariations(conditionalSubstitutions):
     # Overlay
     #
     # Rank is the bit-set of the index of all contributing layers.
-    initMapInit = ((hashdict(),0),) # Initializer representing the entire space
-    boxMap = OrderedDict(initMapInit) # Map from Box to Rank
-    for i,(currRegion,_) in enumerate(conditionalSubstitutions):
+    initMapInit = ((hashdict(), 0),)  # Initializer representing the entire space
+    boxMap = OrderedDict(initMapInit)  # Map from Box to Rank
+    for i, (currRegion, _) in enumerate(conditionalSubstitutions):
         newMap = OrderedDict(initMapInit)
-        currRank = 1<<i
-        for box,rank in boxMap.items():
+        currRank = 1 << i
+        for box, rank in boxMap.items():
             for currBox in currRegion:
                 intersection, remainder = overlayBox(currBox, box)
                 if intersection is not None:
                     intersection = hashdict(intersection)
-                    newMap[intersection] = newMap.get(intersection, 0) | rank|currRank
+                    newMap[intersection] = newMap.get(intersection, 0) | rank | currRank
                 if remainder is not None:
                     remainder = hashdict(remainder)
                     newMap[remainder] = newMap.get(remainder, 0) | rank
         boxMap = newMap
-    del boxMap[hashdict()]
 
     # Generate output
     items = []
-    for box,rank in sorted(boxMap.items(),
-                           key=(lambda BoxAndRank: -popCount(BoxAndRank[1]))):
+    for box, rank in sorted(
+        boxMap.items(), key=(lambda BoxAndRank: -bit_count(BoxAndRank[1]))
+    ):
+        # Skip any box that doesn't have any substitution.
+        if rank == 0:
+            continue
         substsList = []
         i = 0
         while rank:
-          if rank & 1:
-              substsList.append(conditionalSubstitutions[i][1])
-          rank >>= 1
-          i += 1
-        items.append((dict(box),substsList))
+            if rank & 1:
+                substsList.append(conditionalSubstitutions[i][1])
+            rank >>= 1
+            i += 1
+        items.append((dict(box), substsList))
     return items
 
 
@@ -162,12 +242,14 @@ def overlayFeatureVariations(conditionalSubstitutions):
 # from the corresponding axes.
 #
 
+
 def overlayBox(top, bot):
-    """Overlays `top` box on top of `bot` box.
+    """Overlays ``top`` box on top of ``bot`` box.
 
     Returns two items:
-    - Box for intersection of `top` and `bot`, or None if they don't intersect.
-    - Box for remainder of `bot`.  Remainder box might not be exact (since the
+
+    * Box for intersection of ``top`` and ``bot``, or None if they don't intersect.
+    * Box for remainder of ``bot``.  Remainder box might not be exact (since the
       remainder might not be a simple box), but is inclusive of the exact
       remainder.
     """
@@ -182,31 +264,36 @@ def overlayBox(top, bot):
         minimum = max(min1, min2)
         maximum = min(max1, max2)
         if not minimum < maximum:
-            return None, bot # Do not intersect
-        intersection[axisTag] = minimum,maximum
+            return None, bot  # Do not intersect
+        intersection[axisTag] = minimum, maximum
 
     # Remainder
     #
     # Remainder is empty if bot's each axis range lies within that of intersection.
     #
     # Remainder is shrank if bot's each, except for exactly one, axis range lies
-    # within that of intersection, and that one axis, it spills out of the
+    # within that of intersection, and that one axis, it extrudes out of the
     # intersection only on one side.
     #
     # Bot is returned in full as remainder otherwise, as true remainder is not
     # representable as a single box.
 
     remainder = dict(bot)
-    exactlyOne = False
-    fullyInside = False
+    extruding = False
+    fullyInside = True
+    for axisTag in top:
+        if axisTag in bot:
+            continue
+        extruding = True
+        fullyInside = False
+        break
     for axisTag in bot:
-        if axisTag not in intersection:
-            fullyInside = False
-            continue # Axis range lies fully within
+        if axisTag not in top:
+            continue  # Axis range lies fully within
         min1, max1 = intersection[axisTag]
         min2, max2 = bot[axisTag]
         if min1 <= min2 and max2 <= max1:
-            continue # Axis range lies fully within
+            continue  # Axis range lies fully within
 
         # Bot's range doesn't fully lie within that of top's for this axis.
         # We know they intersect, so it cannot lie fully without either; so they
@@ -214,9 +301,9 @@ def overlayBox(top, bot):
 
         # If we have had an overlapping axis before, remainder is not
         # representable as a box, so return full bottom and go home.
-        if exactlyOne:
+        if extruding:
             return intersection, bot
-        exactlyOne = True
+        extruding = True
         fullyInside = False
 
         # Otherwise, cut remainder on this axis and continue.
@@ -232,7 +319,7 @@ def overlayBox(top, bot):
             # Remainder leaks out from both sides.  Can't cut either.
             return intersection, bot
 
-        remainder[axisTag] = minimum,maximum
+        remainder[axisTag] = minimum, maximum
 
     if fullyInside:
         # bot is fully within intersection.  Remainder is empty.
@@ -240,15 +327,16 @@ def overlayBox(top, bot):
 
     return intersection, remainder
 
+
 def cleanupBox(box):
     """Return a sparse copy of `box`, without redundant (default) values.
 
-        >>> cleanupBox({})
-        {}
-        >>> cleanupBox({'wdth': (0.0, 1.0)})
-        {'wdth': (0.0, 1.0)}
-        >>> cleanupBox({'wdth': (-1.0, 1.0)})
-        {}
+    >>> cleanupBox({})
+    {}
+    >>> cleanupBox({'wdth': (0.0, 1.0)})
+    {'wdth': (0.0, 1.0)}
+    >>> cleanupBox({'wdth': (-1.0, 1.0)})
+    {}
 
     """
     return {tag: limit for tag, limit in box.items() if limit != (-1.0, 1.0)}
@@ -258,60 +346,76 @@ def cleanupBox(box):
 # Low level implementation
 #
 
-def addFeatureVariationsRaw(font, conditionalSubstitutions, featureTag='rvrn'):
+
+def addFeatureVariationsRaw(font, table, conditionalSubstitutions, featureTag="rvrn"):
     """Low level implementation of addFeatureVariations that directly
     models the possibilities of the FeatureVariations table."""
 
+    featureTags = [featureTag] if isinstance(featureTag, str) else sorted(featureTag)
+    processLast = "rvrn" not in featureTags or len(featureTags) > 1
+
     #
-    # if there is no <featureTag> feature:
+    # if a <featureTag> feature is not present:
     #     make empty <featureTag> feature
     #     sort features, get <featureTag> feature index
     #     add <featureTag> feature to all scripts
+    # if a <featureTag> feature is present:
+    #     reuse <featureTag> feature index
     # make lookups
     # add feature variations
     #
+    if table.Version < 0x00010001:
+        table.Version = 0x00010001  # allow table.FeatureVariations
 
-    if "GSUB" not in font:
-        font["GSUB"] = buildGSUB()
+    varFeatureIndices = set()
 
-    gsub = font["GSUB"].table
+    existingTags = {
+        feature.FeatureTag
+        for feature in table.FeatureList.FeatureRecord
+        if feature.FeatureTag in featureTags
+    }
 
-    if gsub.Version < 0x00010001:
-        gsub.Version = 0x00010001  # allow gsub.FeatureVariations
+    newTags = set(featureTags) - existingTags
+    if newTags:
+        varFeatures = []
+        for featureTag in sorted(newTags):
+            varFeature = buildFeatureRecord(featureTag, [])
+            table.FeatureList.FeatureRecord.append(varFeature)
+            varFeatures.append(varFeature)
+        table.FeatureList.FeatureCount = len(table.FeatureList.FeatureRecord)
 
-    gsub.FeatureVariations = None  # delete any existing FeatureVariations
+        sortFeatureList(table)
 
-    varFeatureIndices = []
-    for index, feature in enumerate(gsub.FeatureList.FeatureRecord):
-        if feature.FeatureTag == featureTag:
-            varFeatureIndices.append(index)
+        for varFeature in varFeatures:
+            varFeatureIndex = table.FeatureList.FeatureRecord.index(varFeature)
 
-    if not varFeatureIndices:
-        varFeature = buildFeatureRecord(featureTag, [])
-        gsub.FeatureList.FeatureRecord.append(varFeature)
-        gsub.FeatureList.FeatureCount = len(gsub.FeatureList.FeatureRecord)
+            for scriptRecord in table.ScriptList.ScriptRecord:
+                if scriptRecord.Script.DefaultLangSys is None:
+                    raise VarLibError(
+                        "Feature variations require that the script "
+                        f"'{scriptRecord.ScriptTag}' defines a default language system."
+                    )
+                langSystems = [lsr.LangSys for lsr in scriptRecord.Script.LangSysRecord]
+                for langSys in [scriptRecord.Script.DefaultLangSys] + langSystems:
+                    langSys.FeatureIndex.append(varFeatureIndex)
+                    langSys.FeatureCount = len(langSys.FeatureIndex)
+            varFeatureIndices.add(varFeatureIndex)
 
-        sortFeatureList(gsub)
-        varFeatureIndex = gsub.FeatureList.FeatureRecord.index(varFeature)
+    if existingTags:
+        # indices may have changed if we inserted new features and sorted feature list
+        # so we must do this after the above
+        varFeatureIndices.update(
+            index
+            for index, feature in enumerate(table.FeatureList.FeatureRecord)
+            if feature.FeatureTag in existingTags
+        )
 
-        for scriptRecord in gsub.ScriptList.ScriptRecord:
-            langSystems = [lsr.LangSys for lsr in scriptRecord.Script.LangSysRecord]
-            for langSys in [scriptRecord.Script.DefaultLangSys] + langSystems:
-                langSys.FeatureIndex.append(varFeatureIndex)
-
-        varFeatureIndices = [varFeatureIndex]
-
-    # setup lookups
-
-    # turn substitution dicts into tuples of tuples, so they are hashable
-    conditionalSubstitutions, allSubstitutions = makeSubstitutionsHashable(conditionalSubstitutions)
-
-    lookupMap = buildSubstitutionLookups(gsub, allSubstitutions)
-
-    axisIndices = {axis.axisTag: axisIndex for axisIndex, axis in enumerate(font["fvar"].axes)}
+    axisIndices = {
+        axis.axisTag: axisIndex for axisIndex, axis in enumerate(font["fvar"].axes)
+    }
 
     featureVariationRecords = []
-    for conditionSet, substitutions in conditionalSubstitutions:
+    for conditionSet, lookupIndices in conditionalSubstitutions:
         conditionTable = []
         for axisTag, (minValue, maxValue) in sorted(conditionSet.items()):
             if minValue > maxValue:
@@ -320,21 +424,45 @@ def addFeatureVariationsRaw(font, conditionalSubstitutions, featureTag='rvrn'):
                 )
             ct = buildConditionTable(axisIndices[axisTag], minValue, maxValue)
             conditionTable.append(ct)
-
-        lookupIndices = [lookupMap[subst] for subst in substitutions]
         records = []
-        for varFeatureIndex in varFeatureIndices:
-            existingLookupIndices = gsub.FeatureList.FeatureRecord[varFeatureIndex].Feature.LookupListIndex
-            records.append(buildFeatureTableSubstitutionRecord(varFeatureIndex, existingLookupIndices + lookupIndices))
-        featureVariationRecords.append(buildFeatureVariationRecord(conditionTable, records))
+        for varFeatureIndex in sorted(varFeatureIndices):
+            existingLookupIndices = table.FeatureList.FeatureRecord[
+                varFeatureIndex
+            ].Feature.LookupListIndex
+            combinedLookupIndices = (
+                existingLookupIndices + lookupIndices
+                if processLast
+                else lookupIndices + existingLookupIndices
+            )
 
-    gsub.FeatureVariations = buildFeatureVariations(featureVariationRecords)
+            records.append(
+                buildFeatureTableSubstitutionRecord(
+                    varFeatureIndex, combinedLookupIndices
+                )
+            )
+        featureVariationRecords.append(
+            buildFeatureVariationRecord(conditionTable, records)
+        )
+
+    if hasattr(table, "FeatureVariations") and table.FeatureVariations is not None:
+        if table.FeatureVariations.Version != 0x00010000:
+            raise VarLibError(
+                "Unsupported FeatureVariations table version: "
+                f"0x{table.FeatureVariations.Version:08x} (expected 0x00010000)."
+            )
+        table.FeatureVariations.FeatureVariationRecord.extend(featureVariationRecords)
+        table.FeatureVariations.FeatureVariationCount = len(
+            table.FeatureVariations.FeatureVariationRecord
+        )
+    else:
+        table.FeatureVariations = buildFeatureVariations(featureVariationRecords)
 
 
 #
 # Building GSUB/FeatureVariations internals
 #
 
+
 def buildGSUB():
     """Build a GSUB table from scratch."""
     fontTable = newTable("GSUB")
@@ -349,10 +477,11 @@ def buildGSUB():
     gsub.LookupList.Lookup = []
 
     srec = ot.ScriptRecord()
-    srec.ScriptTag = 'DFLT'
+    srec.ScriptTag = "DFLT"
     srec.Script = ot.Script()
     srec.Script.DefaultLangSys = None
     srec.Script.LangSysRecord = []
+    srec.Script.LangSysCount = 0
 
     langrec = ot.LangSysRecord()
     langrec.LangSys = ot.LangSys()
@@ -383,18 +512,51 @@ def makeSubstitutionsHashable(conditionalSubstitutions):
     return condSubst, sorted(allSubstitutions)
 
 
-def buildSubstitutionLookups(gsub, allSubstitutions):
+class ShifterVisitor(TTVisitor):
+    def __init__(self, shift):
+        self.shift = shift
+
+
+@ShifterVisitor.register_attr(ot.Feature, "LookupListIndex")  # GSUB/GPOS
+def visit(visitor, obj, attr, value):
+    shift = visitor.shift
+    value = [l + shift for l in value]
+    setattr(obj, attr, value)
+
+
+@ShifterVisitor.register_attr(
+    (ot.SubstLookupRecord, ot.PosLookupRecord), "LookupListIndex"
+)
+def visit(visitor, obj, attr, value):
+    setattr(obj, attr, visitor.shift + value)
+
+
+def buildSubstitutionLookups(gsub, allSubstitutions, processLast=False):
     """Build the lookups for the glyph substitutions, return a dict mapping
     the substitution to lookup indices."""
-    firstIndex = len(gsub.LookupList.Lookup)
+
+    # Insert lookups at the beginning of the lookup vector
+    # https://github.com/googlefonts/fontmake/issues/950
+
+    firstIndex = len(gsub.LookupList.Lookup) if processLast else 0
     lookupMap = {}
     for i, substitutionMap in enumerate(allSubstitutions):
-        lookupMap[substitutionMap] = i + firstIndex
+        lookupMap[substitutionMap] = firstIndex + i
 
-    for subst in allSubstitutions:
+    if not processLast:
+        # Shift all lookup indices in gsub by len(allSubstitutions)
+        shift = len(allSubstitutions)
+        visitor = ShifterVisitor(shift)
+        visitor.visit(gsub.FeatureList.FeatureRecord)
+        visitor.visit(gsub.LookupList.Lookup)
+
+    for i, subst in enumerate(allSubstitutions):
         substMap = dict(subst)
         lookup = buildLookup([buildSingleSubstSubtable(substMap)])
-        gsub.LookupList.Lookup.append(lookup)
+        if processLast:
+            gsub.LookupList.Lookup.append(lookup)
+        else:
+            gsub.LookupList.Lookup.insert(i, lookup)
         assert gsub.LookupList.Lookup[lookupMap[subst]] is lookup
     gsub.LookupList.LookupCount = len(gsub.LookupList.Lookup)
     return lookupMap
@@ -405,6 +567,7 @@ def buildFeatureVariations(featureVariationRecords):
     fv = ot.FeatureVariations()
     fv.Version = 0x00010000
     fv.FeatureVariationRecord = featureVariationRecords
+    fv.FeatureVariationCount = len(featureVariationRecords)
     return fv
 
 
@@ -423,9 +586,11 @@ def buildFeatureVariationRecord(conditionTable, substitutionRecords):
     fvr = ot.FeatureVariationRecord()
     fvr.ConditionSet = ot.ConditionSet()
     fvr.ConditionSet.ConditionTable = conditionTable
+    fvr.ConditionSet.ConditionCount = len(conditionTable)
     fvr.FeatureTableSubstitution = ot.FeatureTableSubstitution()
     fvr.FeatureTableSubstitution.Version = 0x00010000
     fvr.FeatureTableSubstitution.SubstitutionRecord = substitutionRecords
+    fvr.FeatureTableSubstitution.SubstitutionCount = len(substitutionRecords)
     return fvr
 
 
@@ -435,6 +600,7 @@ def buildFeatureTableSubstitutionRecord(featureIndex, lookupListIndices):
     ftsr.FeatureIndex = featureIndex
     ftsr.Feature = ot.Feature()
     ftsr.Feature.LookupListIndex = lookupListIndices
+    ftsr.Feature.LookupCount = len(lookupListIndices)
     return ftsr
 
 
@@ -453,10 +619,15 @@ def sortFeatureList(table):
     elsewhere. This is needed after the feature list has been modified.
     """
     # decorate, sort, undecorate, because we need to make an index remapping table
-    tagIndexFea = [(fea.FeatureTag, index, fea) for index, fea in enumerate(table.FeatureList.FeatureRecord)]
+    tagIndexFea = [
+        (fea.FeatureTag, index, fea)
+        for index, fea in enumerate(table.FeatureList.FeatureRecord)
+    ]
     tagIndexFea.sort()
     table.FeatureList.FeatureRecord = [fea for tag, index, fea in tagIndexFea]
-    featureRemap = dict(zip([index for tag, index, fea in tagIndexFea], range(len(tagIndexFea))))
+    featureRemap = dict(
+        zip([index for tag, index, fea in tagIndexFea], range(len(tagIndexFea)))
+    )
 
     # Remap the feature indices
     remapFeatures(table, featureRemap)
@@ -479,11 +650,12 @@ def remapFeatures(table, featureRemap):
 
 
 def _remapLangSys(langSys, featureRemap):
-    if langSys.ReqFeatureIndex != 0xffff:
+    if langSys.ReqFeatureIndex != 0xFFFF:
         langSys.ReqFeatureIndex = featureRemap[langSys.ReqFeatureIndex]
     langSys.FeatureIndex = [featureRemap[index] for index in langSys.FeatureIndex]
 
 
 if __name__ == "__main__":
     import doctest, sys
+
     sys.exit(doctest.testmod().failed)
